# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010 Nexedi SA and Contributors. All Rights Reserved.
# Copyright (c) 2006-2008 Zope Corporation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

import os, re, shutil, sys, glob
import zc.buildout
import zc.recipe.egg

class Recipe:
    '''Creates a runUnitTest aware of eggs and custom paths used in this
    buildout.
    '''
    name = 'runUnitTest'

    def __init__(self, buildout, name, options):
        self.buildout, self.options, self.name = buildout, options, name
        
        self.location = options['location'] = os.path.join(
            buildout['buildout']['parts-directory'],
            self.name,
            )
        try:
            self.instance_home = (options.get('instance-home') or
                                  # backward compatibility...
                                  self.buildout['erp5_instance']['location'])
        except KeyError:
            raise RuntimeError('please specify location of ERP5 instance '
                               '(e.g.: "instance-home = '
                               '${erp5_instance:location}")')
        # force the generation of a single script...
        scriptName = options.get('control-script', name).strip()
        options['scripts'] = 'runUnitTest=' + scriptName
        # ...manually defined by our entry point
        options['entry-points'] = '''
          runUnitTest=Products.ERP5Type.tests.runUnitTest:main
        '''
        # initialize the recipe that will actually generate the script
        self.egg = zc.recipe.egg.Egg(buildout, 'recipes', options)

    def install(self):
        options = self.options
        location = self.location

        extra_paths = self.egg.extra_paths
        # insert software-home into extra_paths, calculating from
        # zope2-location if necessary
        software_home = options.get('software-home')
        zope2_location = options.get('zope2-location')
        if zope2_location and not software_home:
            software_home = os.path.join(zope2_location, 'lib', 'python')
        if software_home:
            extra_paths.insert(0, software_home)
            zope_scripts = None
        else:
            # if no software_home can be found, assume software_home is already
            # reachable by eggs or extra-paths (the generated script will check
            # if that is the case), but then the skeletons on
            # ERP5Type/tests/skel2.12 will need to find the Zope startup
            # scripts (zopectl, runzope), which will likely be in this buildout
            # bin directory.
            zope_scripts = options.get('zope2-scripts', 
                                       options['bin-directory'])

        init = init_template % dict(name=self.name,
                                    zope_scripts=zope_scripts)

        products = options.get('products', '')
        test_products = options.get('test-products', products)
        # Add Products to sys.path under the Products namespace
        for product in products.splitlines():
          if product:
            init += add_product_path_template % dict(product_path=product)
        # add test directories of selected products (defaulting to all 
        # products) to sys.path since that's where runUnitTest actually looks
        # for them
        for product in test_products.splitlines():
          if product:
            extra_paths.extend(
                product_test_path for product_test_path in
                glob.glob(os.path.join(product, '*', 'tests'))
            )
            #for product_test_path in glob.glob(
            #                            os.path.join(product, '*', 'tests')):
            #  init += add_sys_path_template % dict(
            #                            sys_path=product_test_path)
              
        for bt5_path in options.get('bt5_path', '').splitlines():
          bt5_path = bt5_path.strip()
          if bt5_path:
            init += add_bt5_path_template % dict(bt5_path=bt5_path)

        # add parts/<instance>/tests to path because runUnitTest will
        # guess it wrong
        local_tests_path = os.path.join(self.instance_home, 'tests')
        init += add_sys_path_template % dict(sys_path=local_tests_path)

        # runUnitTest works in our parts directory, which will contain
        # custom_zodb.py
        if not os.path.exists(location):
          os.mkdir(location)
        init += set_env_and_chdir_template % dict(location=location)

        # framework.py requires a Products directory, so create it here
        # This is temporary, as we're getting rid of "framework.py"
        products_dir = os.path.join(location, 'Products')
        if not os.path.exists(products_dir):
           os.mkdir(products_dir)

        self.options['initialization'] = init
        # reassign just to be on the safe side...
        self.egg.extra_paths = extra_paths

        # generate the script according to our configuration
        self.egg.install()

        return [location]

    update = install

init_template = '''
import os
from os.path import dirname, abspath

# derive SOFTWARE_HOME from import location of Zope2
try:
  import Zope2
except ImportError:
  print >> sys.stderr, ("Zope2 could not be found. In the %(name)r part of the "
                        "buildout configuration, please specify either "
                        "software-home or zope2-location, or add Zope2 to the "
                        "list of eggs.")
  raise
else:
  os.environ['SOFTWARE_HOME'] = abspath(dirname(dirname(Zope2.__file__)))

# if under Zope 2.12, we need the location of the Zope2 startup scripts for the
# substitutions in the skel2.12 directory
ZOPE_SCRIPTS = %(zope_scripts)r
if ZOPE_SCRIPTS is not None:
  os.environ['ZOPE_SCRIPTS'] = ZOPE_SCRIPTS 

import Products

# Testing needs to be imported, otherwise framework.py will guess it from
# sys.path, which can be a zipped egg
import Testing
'''

add_product_path_template = '''
Products.__path__.append(%(product_path)r)'''

add_sys_path_template = '''
sys.path.append(%(sys_path)r)'''

add_bt5_path_template = '''
sys.argv.insert(1, %(bt5_path)r)
sys.argv.insert(1, '--bt5_path')'''

set_env_and_chdir_template = '''

# execute framework.py, like in zope testcase
os.environ['INSTANCE_HOME'] = %(location)r
os.chdir(%(location)r)
'''

